#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <math.h>
#include <sys/stat.h>
#include <pwd.h>
#include <unistd.h>
#include <errno.h>
#include "SDL/SDL.h"
#include "SDL/SDL_endian.h"
#include "SDL/SDL_image.h"
#include "SDL/SDL_mixer.h"
#include "SDL/SDL_ttf.h"
#include "SDL/SDL_net.h"

#define MAX_PARTICLES 1000
#define MAX_ENTITIES 1000
#define MAX_FRAMES 16
#define MAX_STARS 500
#define MAX_INPUTS 5
#define MAX_OFFSET 300
#define MAX_OFFSET_RIGHT 800
#define MAX_PATH 1024
#define MAX_HIGHSCORES 50
#define BUFFER_SIZE 1000
#define MAX_REPLAY_FILES 20
#define MAX_REPLAY_COUNT 100
#define MAX_HIGHSCORE_NAME 23

#define MENU_TIMEOUT 600
#define MENU_DISPLAY_TIMEOUT 30

#define PLAYER_SPEED 5

#define PLAYER_BULLET_SPEED 15
#define ENEMY_BULLET_SPEED 8

#define PI 3.1415926535897932384626433832795

#define SCREEN_WIDTH 800
#define SCREEN_HEIGHT 600

#define RESPAWN_CHANCE 60 * (4 + prand() % 3)

#define REPLAY_BUFFER 1000
#define PACKET_BUFFER 4096

#define ROAD_BLOCK_SPACING 50
#define STANDARD_SPACING 80

#define HIGH_SCORE_SERVER_IP "parallelrealities.co.uk"
#define HIGH_SCORE_SERVER_PORT 80
#define HIGH_SCORE_SERVER_REQUEST_1 "GET /randomShooterScores.php"
#define HIGH_SCORE_SERVER_REQUEST_2 " HTTP/1.1\nHost: parallelrealities.co.uk\nUser-Agent: Random Shooter (Linux; i686)\n\n"
#define HIGH_SCORE_SERVER_RESPONSE_OK "200"

#if SDL_BYTEORDER == SDL_LIL_ENDIAN
	#define SWAP16(X) (X)
	#define SWAP32(X) (X)
#else
	#define SWAP16(X) SDL_Swap16(X)
	#define SWAP32(X) SDL_Swap32(X)
#endif

#ifndef INSTALL_DIR
	#define INSTALL_DIR ""
#endif

#if RELEASE == 0
	#undef INSTALL_DIR
	#define INSTALL_DIR ""
#endif

#ifndef VERSION
	#error VERSION must be defined in the makefile!
#endif

enum
{
	PLAYER,
 	ENEMY,
  	PLAYER_BULLET,
   	ENEMY_BULLET,
	SCORE_POD,
 	SCORE
};

enum
{
	LEFT,
 	RIGHT,
  	UP,
   	DOWN,
	FIRE,
 	PAUSE,
  	MAX_CONTROLS
};

enum
{
	PLAYER_BULLET_SND,
	ENEMY_BULLET_SND,
	BULLET_HIT_SND,
	EXPLOSION_SND,
 	LARGE_EXPLOSION_SND,
	MAX_SOUNDS
};

enum
{
	STRAIGHT_WAVE,
	ROAD_BLOCK_WAVE,
	CURVE_WAVE,
	PARABOLA_WAVE,
	MAX_WAVES
};

enum
{
	MAIN_MENU_START_GAME,
 	MAIN_MENU_REDEFINE,
 	MAIN_MENU_FULLSCREEN,
 	MAIN_MENU_QUIT,
	MAX_MAIN_MENU_ENTRIES
};

enum
{
	CONTROL_MENU_LEFT,
	CONTROL_MENU_RIGHT,
	CONTROL_MENU_UP,
	CONTROL_MENU_DOWN,
	CONTROL_MENU_FIRE,
	CONTROL_MENU_PAUSE,
	CONTROL_MENU_RETURN,
	MAX_CONTROL_MENU_ENTRIES
};

enum
{
	STRING_START_GAME,
	STRING_FULLSCREEN,
  	STRING_REDEFINE,
  	STRING_HELP,
	STRING_QUIT,
 	STRING_SCORE,
  	STRING_HIGH_SCORE,
   	STRING_TITLE,
	STRING_GAME_OVER,
 	STRING_GET_READY,
 	STRING_UP,
 	STRING_DOWN,
 	STRING_LEFT,
 	STRING_RIGHT,
 	STRING_FIRE,
 	STRING_PAUSE,
 	STRING_RETURN,
  	STRING_ONLINE_SCORES,
   	STRING_HEALTH,
   	STRING_NEW_HIGH_SCORE,
	STRING_ENTER_NAME,
 	MAX_STRINGS
};

enum
{
	BLACK,
 	WHITE,
  	RED,
   	YELLOW,
	GREEN,
 	BLUE,
 	CYAN,
 	PURPLE,
  	GREY,
   	DARK_GREY,
	LIGHT_GREY,
  	MAX_COLORS
};

enum
{
	GAME_STATUS_MENU,
 	GAME_STATUS_INTRO,
  	GAME_STATUS_IN_GAME,
   	GAME_STATUS_GAME_OVER,
   	GAME_STATUS_REDEFINE,
   	GAME_STATUS_HELP,
   	GAME_STATUS_HIGHSCORE
};

enum
{
	POINTS_ANIMATION,
 	MENU_BALL_ANIMATION,
  	ENEMY_ANIMATION,
   	PLAYER_ANIMATION,
   	PLAYER_BULLET_ANIMATION,
	ENEMY_BULLET_ANIMATION,
	MAX_ANIMATIONS
};
