#include "enemies.h"

extern long prand(void);
extern int getFreeEntity(void);
extern void fireAimedShot(void);
extern void fire2WayShot(void);
extern void fire4WayShot(void);
extern void fire360Shot(void);
extern void standardMove(void);
extern void dropScorePod(void);
extern void curveMove(void);

void addRandomEnemy()
{
	int i = getFreeEntity();

	if (i == -1)
	{
		return;
	}

	entities[i].type = ENEMY;
	entities[i].x = SCREEN_WIDTH + prand() % 100;
	entities[i].y = prand() % (SCREEN_HEIGHT - 20);
	entities[i].health = 1 + prand() % 10;
	entities[i].dx = -1 * (1 + prand() % 5);
	entities[i].width = 64;
	entities[i].height = 64;
	entities[i].score = 0;

	switch (prand() % 10)
	{
		case 0:
		case 1:
		case 2:
		case 3:
		case 4:
			entities[i].attack = &fireAimedShot;
			entities[i].maxReload = 60 + prand() % 240;
			entities[i].maxAmmo = 1 + prand() % 5;
			entities[i].maxThinkTime = 20 + prand() % 40;
			entities[i].score += 10;
			break;

		case 5:
		case 6:
		case 7:
		case 8:
			entities[i].attack = &fire2WayShot;
			entities[i].maxReload = 60 + prand() % 240;
			entities[i].maxAmmo = (1 + prand() % 5) * 2;
			entities[i].maxThinkTime = 20 + prand() % 40;
			entities[i].score += 25;
			break;
		/*
		case 8:
		case 9:
			entities[i].attack = &fire4WayShot;
			entities[i].maxReload = 60 + prand() % 240;
			entities[i].maxAmmo = (1 + prand() % 5) * 4;
			entities[i].maxThinkTime = 20 + prand() % 40;
			entities[i].score += 50;
			break;
		*/

		default:
			entities[i].attack = &fire360Shot;
			entities[i].maxReload = 5 + prand() % 55;
			entities[i].maxAmmo = 36;
			entities[i].maxThinkTime = 10;
			entities[i].score += 100;
			break;
	}

	if (prand() % 5 == 0)
	{
		entities[i].move = &standardMove;
		entities[i].score += 25;
	}

	else
	{
		entities[i].dy = entities[i].y;
		entities[i].move = &curveMove;
		entities[i].score += 50;
	}

	entities[i].die = &dropScorePod;
	
	entities[i].sprite = animation[ENEMY_ANIMATION];
	entities[i].frameTime = animation[ENEMY_ANIMATION]->frameTime;
	entities[i].frame = 0;
	
	entities[i].width = animation[ENEMY_ANIMATION]->frame[0]->w;
	entities[i].height = animation[ENEMY_ANIMATION]->frame[0]->h;
}

void addWaveEnemy(int x, int y, int movementType, float speed, int attackType)
{
	int i = getFreeEntity();

	if (i == -1)
	{
		return;
	}

	entities[i].type = ENEMY;
	entities[i].x = x;
	entities[i].y = y;
	entities[i].health = 3;
	entities[i].dx = -speed;
	entities[i].width = 64;
	entities[i].height = 64;
	entities[i].score = 0;

	switch (movementType)
	{
		case STRAIGHT_WAVE:
		case ROAD_BLOCK_WAVE:
			entities[i].move = &standardMove;
			entities[i].score += 25;
		break;

		case CURVE_WAVE:
			entities[i].dy = entities[i].y;
			entities[i].move = &curveMove;
			entities[i].score += 50;
		break;
	}

	switch (attackType)
	{
		case 0:
		case 1:
		case 2:
		case 3:
		case 4:
			entities[i].attack = &fireAimedShot;
			entities[i].maxReload = 180;
			entities[i].maxAmmo = 3;
			entities[i].maxThinkTime = 20;
			entities[i].score += 10;
			break;

		case 5:
		case 6:
		case 7:
		case 8:
			entities[i].attack = &fire2WayShot;
			entities[i].maxReload = 180;
			entities[i].maxAmmo = 6;
			entities[i].maxThinkTime = 20;
			entities[i].score += 15;
			break;
		/*
		case 8:
		case 9:
			entities[i].attack = &fire4WayShot;
			entities[i].maxReload = 180;
			entities[i].maxAmmo = 12;
			entities[i].maxThinkTime = 20;
			entities[i].score += 20;
			break;
		*/
			
		default:
			entities[i].attack = &fire360Shot;
			entities[i].maxReload = 180;
			entities[i].maxAmmo = 18;
			entities[i].maxThinkTime = 6;
			entities[i].score += 30;
			break;
	}
	
	entities[i].die = &dropScorePod;
	
	entities[i].sprite = animation[ENEMY_ANIMATION];
	entities[i].frameTime = animation[ENEMY_ANIMATION]->frameTime;
	entities[i].frame = 0;
	
	entities[i].width = animation[ENEMY_ANIMATION]->frame[0]->w;
	entities[i].height = animation[ENEMY_ANIMATION]->frame[0]->h;
}
