#include "font.h"

SDL_Surface *generateString(char *, int, TTF_Font *, SDL_Color);

TTF_Font *loadFont(char *name, int size)
{
	TTF_Font *font = TTF_OpenFont(name, size);

	if (font == NULL)
	{
		printf("Failed to open Font %s: %s\n", name, TTF_GetError());

		exit(1);
	}

	return font;
}

void closeFont(TTF_Font *font)
{
	if (font != NULL)
	{
		TTF_CloseFont(font);
	}
}

void drawString(char *text, int x, int y, int centerX, int centerY, int transparent, TTF_Font *font, SDL_Color foregroundColor)
{
	SDL_Rect src, dest;
	SDL_Surface *surface;

	surface = TTF_RenderUTF8_Shaded(font, text, foregroundColor, blackColor);

	if (surface == NULL)
	{
		printf("Couldn't create String %s: %s\n", text, SDL_GetError());

		return;
	}

	if (transparent == 1)
	{
		SDL_SetColorKey(surface, (SDL_SRCCOLORKEY|SDL_RLEACCEL), colors[BLACK]);
	}

	src.x = 0;
	src.y = 0;
	src.w = surface->w;
	src.h = surface->h;

	dest.x = (centerX == 1 ? (screen->w - surface->w) / 2 : x);
	dest.y = (centerY == 1 ? (screen->h - surface->h) / 2 : y);
	dest.w = surface->w;
	dest.h = surface->h;

	SDL_BlitSurface(surface, &src, screen, &dest);

	SDL_FreeSurface(surface);
}

void drawCachedString(int id, int x, int y, int centerX, int centerY)
{
	SDL_Rect src, dest;

	if (id >= MAX_STRINGS)
	{
		printf("%d larger than MAX_STRINGS!\n", id);

		return;
	}

	if (stringCache.string[id] == NULL)
	{
		printf("String Cache %d is NULL\n", id);

		return;
	}

	src.x = 0;
	src.y = 0;
	src.w = stringCache.string[id]->w;
	src.h = stringCache.string[id]->h;

	dest.x = (centerX == 1 ? (screen->w - stringCache.string[id]->w) / 2 : x);
	dest.y = (centerY == 1 ? (screen->h - stringCache.string[id]->h) / 2 : y);
	dest.w = stringCache.string[id]->w;
	dest.h = stringCache.string[id]->h;

	SDL_BlitSurface(stringCache.string[id], &src, screen, &dest);
}

void initStringCache()
{
	printf("Building String Cache\n");

	stringCache.string[STRING_START_GAME]    = generateString("Start Game", 1, standardFont, whiteColor);
	stringCache.string[STRING_REDEFINE]      = generateString("Redefine Keys", 1, standardFont, whiteColor);
	stringCache.string[STRING_FULLSCREEN]    = generateString("Toggle Fullscreen", 1, standardFont, whiteColor);
	stringCache.string[STRING_HELP]          = generateString("Help", 1, standardFont, whiteColor);
	stringCache.string[STRING_QUIT]          = generateString("Quit", 1, standardFont, whiteColor);

	stringCache.string[STRING_UP]            = generateString("Up", 1, standardFont, whiteColor);
	stringCache.string[STRING_DOWN]          = generateString("Down", 1, standardFont, whiteColor);
	stringCache.string[STRING_LEFT]          = generateString("Left", 1, standardFont, whiteColor);
	stringCache.string[STRING_RIGHT]         = generateString("Right", 1, standardFont, whiteColor);
	stringCache.string[STRING_FIRE]          = generateString("Fire", 1, standardFont, whiteColor);
	stringCache.string[STRING_PAUSE]         = generateString("Pause", 1, standardFont, whiteColor);
	stringCache.string[STRING_RETURN]        = generateString("Return", 1, standardFont, whiteColor);

	stringCache.string[STRING_SCORE]         = generateString("Score", 1, standardFont, whiteColor);
	stringCache.string[STRING_HIGH_SCORE]    = generateString("High Score", 1, standardFont, whiteColor);
	stringCache.string[STRING_HEALTH]        = generateString("Health", 1, standardFont, whiteColor);

	stringCache.string[STRING_TITLE]         = generateString("Random Shooter", 1, largeFont, whiteColor);
	stringCache.string[STRING_GAME_OVER]     = generateString("Game Over", 1, largeFont, whiteColor);
	stringCache.string[STRING_GET_READY]     = generateString("Get Ready!", 1, largeFont, whiteColor);
	stringCache.string[STRING_ONLINE_SCORES] = generateString("Online Scores", 1, largeFont, whiteColor);

	stringCache.string[STRING_NEW_HIGH_SCORE] = generateString("New Highscore!", 1, largeFont, whiteColor);
	stringCache.string[STRING_ENTER_NAME]     = generateString("Name Entry: ", 1, largeFont, whiteColor);
}

void clearStringCache()
{
	int i;

	printf("Emptying String Cache\n");

	for (i=0;i<MAX_STRINGS;i++)
	{
		if (stringCache.string[i] != NULL)
		{
			SDL_FreeSurface(stringCache.string[i]);
		}
	}
}

SDL_Surface *generateString(char *text, int transparent, TTF_Font *font, SDL_Color foregroundColor)
{
	SDL_Surface *surface;

	surface = TTF_RenderUTF8_Shaded(font, text, foregroundColor, blackColor);

	if (surface == NULL)
	{
		printf("Couldn't create String %s: %s\n", text, SDL_GetError());
	}

	else if (transparent == 1)
	{
		SDL_SetColorKey(surface, (SDL_SRCCOLORKEY|SDL_RLEACCEL), colors[BLACK]);
	}

	return surface;
}
