#include "game.h"

extern void doStars(void);
extern void drawGame(void);
extern void getInput(void);
extern void doPlayer(void);
extern void doEntities(void);
extern long prand(void);
extern void createRandomWave(void);
extern void doCollisions(void);
extern void resetStars(void);
extern void doGame(void);
extern void cleanup(void);
extern void resetPlayer(void);
extern void saveReplayData(int);
extern void loadReplayData(void);
extern void loadReplayHeader(void);
extern void saveReplayHeader(void);
extern void setSeed(long);
extern void drawCachedString(int, int, int, int, int);
extern void resetEntities(void);
extern void doParticles(void);
extern void limitFPS(void);
extern void createAutoReplayFile(void);
extern void takeRandomScreenshot(void);

void resetGame()
{
	resetPlayer();
	resetStars();
	resetEntities();

	game.score = 0;
	game.spawnTime = RESPAWN_CHANCE;
	game.throttle = 1;
}

void doIntro()
{
	replayData.version = VERSION;
	replayData.seed = time(NULL);

	replayData.index = 0;

	if (game.recording == 0 && game.replaying == 0)
	{
		createAutoReplayFile();
	}

	else if (game.recording == 1)
	{
		saveReplayHeader();
	}

	else if (game.replaying == 1)
	{
		replayData.index = REPLAY_BUFFER;

		loadReplayHeader();
	}

	setSeed(replayData.seed);

	resetGame();

	while (player.action != NULL)
	{
		getInput();

		doPlayer();
		doStars();

		drawGame();

		if (!(game.replaying == 1 && game.throttle == 0))
		{
			limitFPS();
		}
	}

	game.status = GAME_STATUS_IN_GAME;
}

void doGame()
{
	game.paused = 0;

	while (game.status == GAME_STATUS_IN_GAME || game.status == GAME_STATUS_GAME_OVER)
	{
		getInput();

		if (game.paused == 0)
		{
			if (game.recording == 1 || (game.recording == 0 && game.replaying == 0))
			{
				saveReplayData(0);
			}

			else if (game.replaying == 1)
			{
				loadReplayData();
			}

			game.spawnTime--;

			if (game.spawnTime <= 0)
			{
				createRandomWave();

				game.spawnTime = RESPAWN_CHANCE;
			}

			doPlayer();
			doStars();
			doEntities();
			doParticles();
			doCollisions();

			drawGame();

			if (!(game.replaying == 1 && game.throttle == 0))
			{
				limitFPS();
			}
		}
	}
}
