#include "highscores.h"

extern void drawString(char *, int, int, int, int, int, TTF_Font *, SDL_Color);
extern SDL_Surface *generateString(char *, int, TTF_Font *, SDL_Color);
extern int getSingleInput(void);
extern void limitFPS(void);
extern void doStars(void);
extern void drawStars(void);
extern void drawCachedString(int, int, int, int, int);

void parseServerOutput(char *);
void loadDefaultScores(void);
void createHighScoreSurface(int);
void clearHighScores(void);
void uploadHighScore(void);
void updateHighScores(void);
void enterHighScoreName(void);
int playerHasHighScore(void);
char *encode(char *);

void resetHighScores()
{
	int i;
	
	for (i=0;i<MAX_HIGHSCORES;i++)
	{
		if (highScore[i].nameImage != NULL)
		{
			SDL_FreeSurface(highScore[i].nameImage);

			highScore[i].nameImage = NULL;
		}

		if (highScore[i].scoreImage != NULL)
		{
			SDL_FreeSurface(highScore[i].scoreImage);

			highScore[i].scoreImage = NULL;
		}
		
		highScore[i].score = 0;
		highScore[i].name[0] = '\0';
	}
}

void loadHighScores()
{
	int i;
	char filename[MAX_PATH];
	FILE *fp;

	resetHighScores();

	sprintf(filename, "%shighscores.dat", game.savePath);

	fp = fopen(filename, "rb");

	if (fp == NULL)
	{
		loadDefaultScores();

		return;
	}

	for (i=0;i<MAX_HIGHSCORES;i++)
	{
		if (fread(&highScore[i].name, sizeof(highScore[i].name), 1, fp) == 1)
		{
			if (fread(&highScore[i].score, sizeof(highScore[i].score), 1, fp) == 1)
			{
				createHighScoreSurface(i);
			}
		}
	}

	fclose(fp);
}

void saveHighScores()
{
	int i;
	char filename[MAX_PATH];
	FILE *fp;

	sprintf(filename, "%shighscores.dat", game.savePath);

	fp = fopen(filename, "wb");

	if (fp == NULL)
	{
		return;
	}

	for (i=0;i<MAX_HIGHSCORES;i++)
	{
		fwrite(&highScore[i].name, sizeof(highScore[i].name), 1, fp);
		fwrite(&highScore[i].score, sizeof(highScore[i].score), 1, fp);
	}

	fclose(fp);

	resetHighScores();
}

void downloadHighScores()
{
	IPaddress ip;
	TCPsocket sd;
	int len;
	char buffer[PACKET_BUFFER];

	if (SDLNet_ResolveHost(&ip, HIGH_SCORE_SERVER_IP, HIGH_SCORE_SERVER_PORT) < 0)
	{
		printf("SDLNet_ResolveHost: %s\n", SDLNet_GetError());

		return;
	}

	if (!(sd = SDLNet_TCP_Open(&ip)))
	{
		printf("SDLNet_TCP_Open: %s\n", SDLNet_GetError());

		return;
	}

	strcpy(buffer, HIGH_SCORE_SERVER_REQUEST_1);
	strcat(buffer, HIGH_SCORE_SERVER_REQUEST_2);

	len = strlen(buffer) + 1;

	SDLNet_TCP_Send(sd, buffer, len);
	SDLNet_TCP_Recv(sd, buffer, BUFFER_SIZE);

	SDLNet_TCP_Close(sd);

	parseServerOutput(buffer);
}

void showHighScores()
{
	int i, start, displayTimer, pauseTimer, moreScores;
	SDL_Rect src, dest;

	start = 0;
	displayTimer = 300;
	pauseTimer = 30;
	moreScores = 1;

	while (start < MAX_HIGHSCORES)
	{
		while (displayTimer > 0)
		{
			if ((getSingleInput() != -1 && pauseTimer <= 0))
			{
				return;
			}

			SDL_FillRect(screen, NULL, colors[BLACK]);

			doStars();

			drawStars();

			drawCachedString(STRING_ONLINE_SCORES, 25, 10, 1, 0);

			if (pauseTimer <= 0)
			{
				for (i=start;i<start + 10;i++)
				{
					if (highScore[i].nameImage != NULL && highScore[i].scoreImage != NULL)
					{
						src.x = 0;
						src.y = 0;
						src.w = highScore[i].nameImage->w;
						src.h = highScore[i].nameImage->h;

						dest.x = 250;
						dest.y = 80 + (i - start) * 50;
						dest.w = highScore[i].nameImage->w;
						dest.h = highScore[i].nameImage->h;

						SDL_BlitSurface(highScore[i].nameImage, &src, screen, &dest);

						src.x = 0;
						src.y = 0;
						src.w = highScore[i].scoreImage->w;
						src.h = highScore[i].scoreImage->h;

						dest.x = 475;
						dest.y = 80 + (i - start) * 50;
						dest.w = highScore[i].scoreImage->w;
						dest.h = highScore[i].scoreImage->h;

						SDL_BlitSurface(highScore[i].scoreImage, &src, screen, &dest);
					}

					else
					{
						moreScores = 0;
					}
				}
			}

			SDL_Flip(screen);

			limitFPS();

			displayTimer--;

			pauseTimer--;
		}

		if (moreScores == 0)
		{
			return;
		}

		start += 10;

		displayTimer = 300;

		pauseTimer = 30;
	}
}

void parseServerOutput(char *buffer)
{
	int i;
	char *line, *score, *date, *name;

	line = strtok(buffer, "\n");

	if (strstr(line, HIGH_SCORE_SERVER_RESPONSE_OK) == NULL)
	{
		printf("Server returned error: %s\n", line);

		return;
	}

	while (line != NULL)
	{
		if (strlen(line) == 1)
		{
			line = strtok(NULL, "\n");
			line = strtok(NULL, "\n");

			break;
		}

		line = strtok(NULL, "\n");
	}

	resetHighScores();

	i = 0;

	score = strtok(NULL, "|");
	name = strtok(NULL, "|");
	date = strtok(NULL, "\n");

	while (name != NULL && i < MAX_HIGHSCORES)
	{
		strncpy(highScore[i].name, name, MAX_HIGHSCORE_NAME - 1);

		highScore[i].score = atoi(score);

		createHighScoreSurface(i);

		score = strtok(NULL, "|");
		name = strtok(NULL, "|");
		date = strtok(NULL, "\n");

		i++;
	}
	
	game.highScore = highScore[0].score;
}

void loadDefaultScores()
{
	int i;
	
	resetHighScores();

	for (i=0;i<MAX_HIGHSCORES;i++)
	{
		sprintf(highScore[i].name, "Player %2d", (i + 1));
		highScore[i].score = (MAX_HIGHSCORES - i) * 20;

		createHighScoreSurface(i);
	}
}

void createHighScoreSurface(int i)
{
	char text[60];
	
	if (highScore[i].score == 0)
	{
		return;
	}

	sprintf(text, "%2d. %s", (i + 1), highScore[i].name);

	highScore[i].nameImage = generateString(text, 1, standardFont, whiteColor);

	sprintf(text, "%9d", highScore[i].score);

	highScore[i].scoreImage = generateString(text, 1, standardFont, whiteColor);

	if (highScore[i].nameImage == NULL || highScore[i].scoreImage == NULL)
	{
		printf("Failed to create image for Highscore %d\n", i);

		exit(0);
	}
}

void doHighScoreCheck()
{
	if (playerHasHighScore() == 1)
	{
		enterHighScoreName();
	}

	game.status = GAME_STATUS_MENU;
}

int playerHasHighScore()
{
	int i;

	for (i=0;i<MAX_HIGHSCORES;i++)
	{
		if (highScore[i].score < game.score)
		{
			return 1;
		}
	}

	return 0;
}

void enterHighScoreName()
{
	int key, go, i, timer;
	char score[20];

	go = 1;
	i = strlen(game.playerName);
	
	strcat(game.playerName, "_");
	
	sprintf(score, "Score: %d", game.score);
	
	timer = 180;

	while (go == 1)
	{
		timer--;
		
		if (timer < 0)
		{
			timer = 0;
		}
		
		key = getSingleInput();

		if (timer <= 0)
		{
			switch (key)
			{
				case SDLK_BACKSPACE:
					if (i > 0)
					{
						i--;
	
						game.playerName[i] = '_';
						game.playerName[i + 1] = '\0';
					}
				break;
	
				case SDLK_RETURN:
					if (strlen(game.playerName) > 1)
					{
						go = 0;
					}
				break;
	
				default:
					if (i < MAX_HIGHSCORE_NAME - 3)
					{
						if (isalnum(key) || key == SDLK_SPACE)
						{
							game.playerName[i] = toupper(key);
							
							i++;
	
							game.playerName[i] = '_';
							game.playerName[i + 1] = '\0';
						}
					}
				break;
			}
		}
		
		SDL_FillRect(screen, NULL, colors[BLACK]);

		doStars();

		drawStars();
		
		if (timer <= 120)
		{
			drawCachedString(STRING_NEW_HIGH_SCORE, 25, 100, 1, 0);
		}
		
		if (timer <= 60)
		{
			drawString(score, 200, 200, 1, 0, 1, largeFont, whiteColor);
		}
		
		if (timer <= 0)
		{
			drawCachedString(STRING_ENTER_NAME, 25, 300, 1, 0);
			drawString(game.playerName, 350, 400, 1, 0, 1, largeFont, whiteColor);
		}

		limitFPS();

		SDL_Flip(screen);
	}
	
	if (strlen(game.playerName) > 1)
	{
		game.playerName[strlen(game.playerName) - 1] = '\0';
				
		updateHighScores();
	}
}

void updateHighScores()
{
	int i, index;

	index = -1;

	for (i=MAX_HIGHSCORES-1;i>=0;i--)
	{
		if (highScore[i].score < game.score)
		{
			if (i - 1 > 0)
			{
				if (highScore[i - 1].score >= game.score)
				{
					index = i;

					break;
				}
			}

			else
			{
				index = 0;

				break;
			}
		}
	}

	if (index != -1)
	{
		for (i=0;i<MAX_HIGHSCORES;i++)
		{
			if (highScore[i].nameImage != NULL)
			{
				SDL_FreeSurface(highScore[i].nameImage);
				
				highScore[i].nameImage = NULL;
			}

			if (highScore[i].scoreImage != NULL)
			{
				SDL_FreeSurface(highScore[i].scoreImage);
				
				highScore[i].scoreImage = NULL;
			}
		}

		for (i=MAX_HIGHSCORES-2;i>=index;i--)
		{
			highScore[i + 1].score = highScore[i].score;
			strcpy(highScore[i + 1].name, highScore[i].name);
		}

		highScore[index].score = game.score;
		strcpy(highScore[index].name, game.playerName);

		for (i=0;i<MAX_HIGHSCORES;i++)
		{
			createHighScoreSurface(i);
		}

		uploadHighScore();
	}
}

void uploadHighScore()
{
	/* You'll need to provide your own implementation of this function */
}

char *encode(char *text)
{
	int i, length;
	char *encoded, code[5];
	
	length = strlen(text);
	
	encoded = (char *)malloc(length * 4 + 1);
	
	encoded[0] = '\0';
	
	for (i=0;i<length;i++)
	{
		sprintf(code, "%%%x", (int)text[i]);
		
		strcat(encoded, code);
	}
	
	return encoded;
}
