#include "input.h"

int getValueFromInput(int, int);

void getInput()
{
	SDL_Event event;

	memset(config.input, 0, sizeof(config.input));

	while (SDL_PollEvent(&event))
	{
		switch (event.type)
		{
			case SDL_KEYDOWN:
				keyState[event.key.keysym.sym] = 1;
			break;

			case SDL_KEYUP:
				keyState[event.key.keysym.sym] = 0;
			break;

			case SDL_JOYBUTTONDOWN:
				buttonState[event.jbutton.button] = 1;
				break;

			case SDL_JOYBUTTONUP:
				buttonState[event.jbutton.button] = 0;
				break;

			case SDL_JOYAXISMOTION:
				if (event.jaxis.axis == 0)
				{
					userDefined.joyX = event.jaxis.value;
				}

				if (event.jaxis.axis == 1)
				{
					userDefined.joyY = event.jaxis.value;
				}

				break;

			case SDL_QUIT:
				exit(0);
			break;
		}
	}

	if (keyState[SDLK_ESCAPE] == 1)
	{
		if (game.status == GAME_STATUS_IN_GAME)
		{
			player.action = player.die;
		}

		else
		{
			exit(0);
		}
	}

	if (keyState[SDLK_F10] == 1)
	{
		game.throttle = 1 - game.throttle;

		keyState[SDLK_F10] = 0;
	}

	if (keyState[SDLK_F12] == 1 || (keyState[SDLK_LALT] == 1 && keyState[SDLK_RETURN] == 1))
	{
		SDL_WM_ToggleFullScreen(screen);

		game.fullscreen = 1 - game.fullscreen;

		keyState[SDLK_RETURN] = keyState[SDLK_LALT] = keyState[SDLK_F12] = 0;
	}

	config.input[LEFT] = keyState[userDefined.input[LEFT]];
	config.input[RIGHT] = keyState[userDefined.input[RIGHT]];
	config.input[UP] = keyState[userDefined.input[UP]];
	config.input[DOWN] = keyState[userDefined.input[DOWN]];

	if (userDefined.joyX < -3200) config.input[LEFT] = 1;
	if (userDefined.joyX > 3200) config.input[RIGHT] = 1;

	if (userDefined.joyY < -3200) config.input[UP] = 1;
	if (userDefined.joyY > 3200) config.input[DOWN] = 1;

	config.input[FIRE] = getValueFromInput(userDefined.input[FIRE], 0);

	if (getValueFromInput(userDefined.input[PAUSE], 1) == 1)
	{
		if (game.status == GAME_STATUS_IN_GAME)
		{
			if (game.paused == 1)
			{
				game.paused = 0;

				Mix_ResumeMusic();
			}

			else
			{
				game.paused = 1;

				Mix_PauseMusic();
			}
		}
	}
}

int getSingleInput()
{
	int key = -1;
	SDL_Event event;

	while (SDL_PollEvent(&event))
	{
		switch (event.type)
		{
			case SDL_KEYDOWN:
				key = event.key.keysym.sym;
			break;

			case SDL_JOYBUTTONDOWN:
				key = event.jbutton.button;
			break;

			case SDL_QUIT:
				exit(0);
			break;
		}
	}

	return key;
}

int getValueFromInput(int inputValue, int reset)
{
	int state = 0;

	if (inputValue >= 0)
	{
		if (inputValue < sizeof(keyState))
		{
			state = keyState[inputValue];

			if (reset == 1 && state == 1)
			{
				keyState[inputValue] = 0;
			}
		}

		if (buttonState != NULL && inputValue < SDL_JoystickNumButtons(joystick))
		{
			state = buttonState[inputValue];

			if (reset == 1 && state == 1)
			{
				buttonState[inputValue] = 0;
			}
		}
	}

	return state;
}

char *getKeyName(int inputValue)
{
	char *name = (char *)malloc(25);

	strncpy(name, SDL_GetKeyName(inputValue), 24);

	if (strcmp(name, "unknown key") == 0)
	{
		sprintf(name, "Joystick Button %d", inputValue);
	}

	return name;
}
