#include "main.h"

extern void init(void);
extern void doIntro(void);
extern void cleanup(void);
extern void doGame(void);
extern void doMainMenu(void);
extern void initMenus(void);
extern Animation *loadAnimation(char *, int);
extern void doControlMenu(void);
extern void loadHighScores(void);
extern void downloadHighScores(void);
extern void doHighScoreCheck(void);

int main(int argc, char *argv[])
{
	int i;

	init();

	atexit(cleanup);

	game.status = GAME_STATUS_MENU;

	for (i=0;i<argc;i++)
	{
		if (strcmp(argv[i], "-record") == 0)
		{
			game.recording = 1;

			i++;

			if (i < argc)
			{
				strncpy(replayData.filename, argv[i], MAX_PATH - 1);
			}

			else
			{
				printf("-record requires a filename to record to\n");

				exit(0);
			}
		}

		if (strcmp(argv[i], "-playback") == 0)
		{
			game.replaying = 1;

			i++;

			if (i < argc)
			{
				strncpy(replayData.filename, argv[i], MAX_PATH - 1);
			}

			else
			{
				printf("-playback requires a filename to playback from\n");

				exit(0);
			}
		}
	}

	if (game.recording == 1)
	{
		replayFile = fopen(replayData.filename, "wb");

		if (replayFile == NULL)
		{
			printf("Could not open %s for recording\n", replayData.filename);

			exit(0);
		}

		game.status = GAME_STATUS_INTRO;
	}

	else if (game.replaying == 1)
	{
		replayFile = fopen(replayData.filename, "rb");

		if (replayFile == NULL)
		{
			printf("Could not open %s for replaying\n", replayData.filename);

			exit(0);
		}

		game.status = GAME_STATUS_INTRO;
	}

	animation[POINTS_ANIMATION] = loadAnimation("points_frame", 2);
	animation[MENU_BALL_ANIMATION] = loadAnimation("menu_ball", 3);
	animation[ENEMY_ANIMATION] = loadAnimation("enemy_frame", 2);
	animation[PLAYER_BULLET_ANIMATION] = loadAnimation("player_bullet", 60);
	animation[ENEMY_BULLET_ANIMATION] = loadAnimation("enemy_bullet", 60);
	animation[PLAYER_ANIMATION] = loadAnimation("player", 60);

	loadHighScores();

	initMenus();

	if (game.online == 1 && game.recording == 0 && game.replaying == 0)
	{
		downloadHighScores();
	}

	while (1)
	{
		switch (game.status)
		{
			case GAME_STATUS_MENU:
				doMainMenu();
				break;

			case GAME_STATUS_REDEFINE:
				doControlMenu();
				break;

			case GAME_STATUS_INTRO:
				doIntro();
				break;

			case GAME_STATUS_IN_GAME:
				doGame();
				break;

			case GAME_STATUS_HIGHSCORE:
				doHighScoreCheck();
			break;

			default:
				exit(0);
				break;
		}
	}

	return 0;
}
