#include "menu.h"

extern void resetStars(void);
extern void getInput(void);
extern void doStars(void);
extern void drawStars(void);
extern void drawCachedString(int, int, int, int, int);
extern void doIntro(void);
extern char *getKeyName(int);
extern int getSingleInput(void);
extern void limitFPS(void);
extern void drawString(char *, int, int, int, int, int, TTF_Font *, SDL_Color);

void processMainMenuInput(int *);
void processControlMenuInput(void);
void drawMainMenu(int *);
void drawControlMenu(int *);
void showHighScores(void);

void initMenus()
{
	mainMenu.entryValue = (int *)malloc(sizeof(int) * MAX_MAIN_MENU_ENTRIES);
	controlMenu.entryValue = (int *)malloc(sizeof(int) * MAX_CONTROL_MENU_ENTRIES);

	if (mainMenu.entryValue == NULL || controlMenu.entryValue == NULL)
	{
		printf("Could not build menus!\n");

		exit(1);
	}

	mainMenu.entryValue[MAIN_MENU_START_GAME] = STRING_START_GAME;
	mainMenu.entryValue[MAIN_MENU_REDEFINE]   = STRING_REDEFINE;
	mainMenu.entryValue[MAIN_MENU_FULLSCREEN] = STRING_FULLSCREEN;
	mainMenu.entryValue[MAIN_MENU_QUIT]       = STRING_QUIT;

	controlMenu.entryValue[CONTROL_MENU_UP]     = STRING_UP;
	controlMenu.entryValue[CONTROL_MENU_DOWN]   = STRING_DOWN;
	controlMenu.entryValue[CONTROL_MENU_LEFT]   = STRING_LEFT;
	controlMenu.entryValue[CONTROL_MENU_RIGHT]  = STRING_RIGHT;
	controlMenu.entryValue[CONTROL_MENU_FIRE]   = STRING_FIRE;
	controlMenu.entryValue[CONTROL_MENU_PAUSE]  = STRING_PAUSE;
	controlMenu.entryValue[CONTROL_MENU_RETURN] = STRING_RETURN;

	mainMenu.sprite = animation[MENU_BALL_ANIMATION];
	mainMenu.frameTime = animation[MENU_BALL_ANIMATION]->frameTime;
	mainMenu.frame = 0;

	controlMenu.sprite = animation[MENU_BALL_ANIMATION];
	controlMenu.frameTime = animation[MENU_BALL_ANIMATION]->frameTime;
	controlMenu.frame = 0;

	resetStars();
}

void doMainMenu()
{
	int scoreTimer = MENU_TIMEOUT;
	int displayTimer = MENU_DISPLAY_TIMEOUT;

	while (game.status == GAME_STATUS_MENU)
	{
		getInput();

		doStars();

		if (displayTimer <= 0)
		{
			processMainMenuInput(&scoreTimer);

			mainMenu.frameTime--;

			if (mainMenu.frameTime <= 0)
			{
				mainMenu.frame++;

				mainMenu.frameTime = mainMenu.sprite->frameTime;

				if (mainMenu.frame >= MAX_FRAMES || mainMenu.frame >= mainMenu.sprite->frames)
				{
					mainMenu.frame = 0;
				}

			}
		}

		drawMainMenu(&displayTimer);

		limitFPS();

		scoreTimer--;

		displayTimer--;

		if (scoreTimer <= 0)
		{
			showHighScores();

			displayTimer = MENU_DISPLAY_TIMEOUT;

			scoreTimer = MENU_TIMEOUT;
		}
	}
}

void doControlMenu()
{
	int displayTimer = MENU_DISPLAY_TIMEOUT;

	while (game.status == GAME_STATUS_REDEFINE)
	{
		getInput();

		doStars();

		if (displayTimer <= 0)
		{
			processControlMenuInput();

			controlMenu.frameTime--;

			if (controlMenu.frameTime <= 0)
			{
				controlMenu.frame++;

				controlMenu.frameTime = controlMenu.sprite->frameTime;

				if (controlMenu.frame >= MAX_FRAMES || controlMenu.frame >= controlMenu.sprite->frames)
				{
					controlMenu.frame = 0;
				}
			}
		}

		drawControlMenu(&displayTimer);

		displayTimer--;

		limitFPS();
	}
}

void processMainMenuInput(int *timer)
{
	if (keyState[SDLK_DOWN] == 1 || config.input[DOWN] == 1)
	{
		mainMenu.selectedEntry++;

		if (mainMenu.selectedEntry >= MAX_MAIN_MENU_ENTRIES)
		{
			mainMenu.selectedEntry = 0;
		}

		config.input[DOWN] = keyState[SDLK_DOWN] = 0;
		
		*timer = MENU_TIMEOUT;
	}

	if (keyState[SDLK_UP] == 1 || config.input[UP] == 1)
	{
		mainMenu.selectedEntry--;

		if (mainMenu.selectedEntry < 0)
		{
			mainMenu.selectedEntry = MAX_MAIN_MENU_ENTRIES - 1;
		}

		config.input[DOWN] = keyState[SDLK_UP] = 0;
		
		*timer = MENU_TIMEOUT;
	}

	if (keyState[SDLK_SPACE] == 1 || keyState[SDLK_RETURN] == 1 || config.input[FIRE] == 1)
	{
		config.input[FIRE] = keyState[SDLK_SPACE] = keyState[SDLK_RETURN] = 0;

		switch (mainMenu.selectedEntry)
		{
			case MAIN_MENU_START_GAME:
				game.status = GAME_STATUS_INTRO;
			break;

			case MAIN_MENU_REDEFINE:
				game.status = GAME_STATUS_REDEFINE;
			break;

			case MAIN_MENU_FULLSCREEN:
				game.fullscreen = 1 - game.fullscreen;

				SDL_WM_ToggleFullScreen(screen);
			break;

			case MAIN_MENU_QUIT:
				exit(0);
			break;
		}
		
		*timer = MENU_TIMEOUT;
	}
}

void processControlMenuInput()
{
	int key = -1, oldKey, timer;

	timer = 0;

	if (keyState[SDLK_DOWN] == 1 || config.input[DOWN] == 1)
	{
		controlMenu.selectedEntry++;

		if (controlMenu.selectedEntry >= MAX_CONTROL_MENU_ENTRIES)
		{
			controlMenu.selectedEntry = 0;
		}

		config.input[DOWN] = keyState[SDLK_DOWN] = 0;
	}

	if (keyState[SDLK_UP] == 1 || config.input[UP] == 1)
	{
		controlMenu.selectedEntry--;

		if (controlMenu.selectedEntry < 0)
		{
			controlMenu.selectedEntry = MAX_CONTROL_MENU_ENTRIES - 1;
		}

		config.input[UP] = keyState[SDLK_UP] = 0;
	}

	if (keyState[SDLK_SPACE] == 1 || keyState[SDLK_RETURN] == 1 || config.input[FIRE] == 1)
	{
		config.input[FIRE] = keyState[SDLK_SPACE] = keyState[SDLK_RETURN] = 0;

		switch (controlMenu.selectedEntry)
		{
			case CONTROL_MENU_RETURN:
				game.status = GAME_STATUS_MENU;
			break;

			default:
				oldKey = userDefined.input[controlMenu.selectedEntry];

				userDefined.input[controlMenu.selectedEntry] = SDLK_QUESTION;

				while (key == -1)
				{
					key = getSingleInput();

					doStars();

					drawControlMenu(&timer);

					limitFPS();
				}

				userDefined.input[controlMenu.selectedEntry] = (key == SDLK_ESCAPE ? oldKey : key);
			break;
		}
	}
}

void drawMainMenu(int *displayTimer)
{
	int i;
	SDL_Rect src, dest;

	SDL_FillRect(screen, NULL, colors[BLACK]);

	drawStars();

	if (*displayTimer <= 0)
	{
		src.x = 0;
		src.y = 0;
		src.w = mainMenu.sprite->frame[mainMenu.frame]->w;
		src.h = mainMenu.sprite->frame[mainMenu.frame]->h;

		dest.x = 250;
		dest.y = 185 + (mainMenu.selectedEntry * 50);
		dest.w = mainMenu.sprite->frame[mainMenu.frame]->w;
		dest.h = mainMenu.sprite->frame[mainMenu.frame]->h;

		SDL_BlitSurface(mainMenu.sprite->frame[mainMenu.frame], &src, screen, &dest);

		drawCachedString(STRING_TITLE, 25, 10, 1, 0);

		for (i=0;i<MAX_MAIN_MENU_ENTRIES;i++)
		{
			drawCachedString(mainMenu.entryValue[i], 25, 200 + (i * 50), 1, 0);
		}
	}

	SDL_Flip(screen);
}

void drawControlMenu(int *displayTimer)
{
	SDL_Rect src, dest;

	SDL_FillRect(screen, NULL, colors[BLACK]);

	drawStars();

	if (*displayTimer <= 0)
	{
		src.x = 0;
		src.y = 0;
		src.w = controlMenu.sprite->frame[controlMenu.frame]->w;
		src.h = controlMenu.sprite->frame[controlMenu.frame]->h;

		dest.x = 95;
		dest.y = 135 + (controlMenu.selectedEntry * 50);
		dest.w = controlMenu.sprite->frame[controlMenu.frame]->w;
		dest.h = controlMenu.sprite->frame[controlMenu.frame]->h;

		if (controlMenu.selectedEntry == CONTROL_MENU_RETURN)
		{
			dest.y += 25;
		}

		SDL_BlitSurface(controlMenu.sprite->frame[controlMenu.frame], &src, screen, &dest);

		drawCachedString(controlMenu.entryValue[CONTROL_MENU_LEFT], 150, 150, 0, 0);
		drawString(getKeyName(userDefined.input[LEFT]), 300, 150, 0, 0, 1, standardFont, whiteColor);

		drawCachedString(controlMenu.entryValue[CONTROL_MENU_RIGHT], 150, 200, 0, 0);
		drawString(getKeyName(userDefined.input[RIGHT]), 300, 200, 0, 0, 1, standardFont, whiteColor);

		drawCachedString(controlMenu.entryValue[CONTROL_MENU_UP], 150, 250, 0, 0);
		drawString(getKeyName(userDefined.input[UP]), 300, 250, 0, 0, 1, standardFont, whiteColor);

		drawCachedString(controlMenu.entryValue[CONTROL_MENU_DOWN], 150, 300, 0, 0);
		drawString(getKeyName(userDefined.input[DOWN]), 300, 300, 0, 0, 1, standardFont, whiteColor);

		drawCachedString(controlMenu.entryValue[CONTROL_MENU_FIRE], 150, 350, 0, 0);
		drawString(getKeyName(userDefined.input[FIRE]), 300, 350, 0, 0, 1, standardFont, whiteColor);

		drawCachedString(controlMenu.entryValue[CONTROL_MENU_PAUSE], 150, 400, 0, 0);
		drawString(getKeyName(userDefined.input[PAUSE]), 300, 400, 0, 0, 1, standardFont, whiteColor);

		drawCachedString(controlMenu.entryValue[CONTROL_MENU_RETURN], 25, 475, 1, 0);
	}

	SDL_Flip(screen);
}

void freeMenus()
{
	if (mainMenu.entryValue != NULL)
	{
		free(mainMenu.entryValue);
	}

	if (controlMenu.entryValue != NULL)
	{
		free(controlMenu.entryValue);
	}
}
