#include "player.h"

extern void addBullet(float, float, int);
extern void playSound(int);
extern long prand(void);
extern void saveReplayData(int);

void playerDie(void);
void flyOnToScreen(void);
void moveBackToStart(void);
void calculatePathToTarget(Entity *, float, float);
void addRandomParticle(float, float);

void resetPlayer()
{
	memset(&player, 0, sizeof(Entity));

	player.width = 25;
	player.height = 14;
	player.health = 1;
	player.active = 1;

	player.maxReload = 5;

	player.x = -100;
	player.y = SCREEN_HEIGHT / 2;

	player.targetX = 100;
	player.targetY = SCREEN_HEIGHT / 2;

	calculatePathToTarget(&player, player.targetX, player.targetY);

	player.dx *= PLAYER_SPEED;
	player.dy *= PLAYER_SPEED;

	player.action = &flyOnToScreen;
	player.die = &playerDie;

	player.sprite = animation[PLAYER_ANIMATION];
	player.frameTime = animation[PLAYER_ANIMATION]->frameTime;
	player.frame = 0;
}

void doPlayer()
{
	self = &player;

	if (self->action != NULL)
	{
		self->action();

		return;
	}

	if (config.input[LEFT] == 1)
	{
		self->x -= self->dx;

		if (self->x < 0)
		{
			self->x = 0;
		}
	}

	if (config.input[RIGHT] == 1)
	{
		self->x += self->dx;

		if (self->x > SCREEN_WIDTH - player.width)
		{
			self->x = SCREEN_WIDTH - player.width;
		}
	}

	if (config.input[UP] == 1)
	{
		self->y -= self->dy;

		if (self->y < 0)
		{
			self->y = 0;
		}
	}

	if (config.input[DOWN] == 1)
	{
		self->y += self->dy;

		if (self->y > SCREEN_HEIGHT - player.height)
		{
			self->y = SCREEN_HEIGHT - player.height;
		}
	}

	if (config.input[FIRE] == 1)
	{
		if (self->reload <= 0)
		{
			playSound(PLAYER_BULLET_SND);

			addBullet(self->x + 100, self->y + self->height / 2, PLAYER_BULLET_SPEED);

			self->reload = self->maxReload;
		}
	}

	self->reload--;
}

void playerDie()
{
	int i;

	if (player.health > 0)
	{
		player.health--;

		if (rand() % 3 == 0)
		{
			addRandomParticle(player.x + player.width / 2, player.y + player.height / 2);
		}

		if (player.health <= 0)
		{
			for (i=0;i<MAX_PARTICLES;i++)
			{
				addRandomParticle(player.x + player.width / 2, player.y + player.height / 2);
			}

			playSound(LARGE_EXPLOSION_SND);

			player.thinkTime = 180;

			if (game.score > game.highScore && game.replaying == 0)
			{
				game.highScore = game.score;
			}
		}
	}

	else if (player.thinkTime > 0)
	{
		player.thinkTime--;

		if (player.thinkTime == 0)
		{
			player.action = NULL;

			if ((game.recording == 0 && game.replaying == 0) || game.recording == 1)
			{
				saveReplayData(1);
			}

			if (game.recording == 1 || game.replaying == 1)
			{
				printf("Final Score: %d\n", game.score);

				exit(0);
			}

			game.status = GAME_STATUS_HIGHSCORE;
		}
	}
}

void drawPlayer()
{
	SDL_Rect src, dest;

	src.x = player.x;
	src.y = player.y;
	src.w = player.width;
	src.h = player.height;

	if (player.health > 0)
	{
		if (player.sprite != NULL)
		{
			src.x = 0;
			src.y = 0;
			src.w = player.sprite->frame[player.frame]->w;
			src.h = player.sprite->frame[player.frame]->h;

			dest.x = player.x;
			dest.y = player.y;
			dest.w = player.sprite->frame[player.frame]->w;
			dest.h = player.sprite->frame[player.frame]->h;

			SDL_BlitSurface(player.sprite->frame[player.frame], &src, screen, &dest);
		}

		else
		{
			printf("Couldn't draw player!\n");

			exit(0);
		}
	}
}

void flyOnToScreen()
{
	player.x += player.dx;
	player.y += player.dy;

	if (fabs(player.x - player.targetX) <= fabs(player.dx) && fabs(player.y - player.targetY) <= fabs(player.dy))
	{
		player.dx = player.dy = PLAYER_SPEED;

		player.action = NULL;
	}
}

