#include "replay.h"

void saveReplayHeader(void);

void createAutoReplayFile()
{
	FILE *fp;

	sprintf(replayData.filename, "%sreplay%03d.dat", game.savePath, (replayData.fileIndex + 1));

	fp = fopen(replayData.filename, "wb");

	replayFile = fp;

	replayData.fileIndex++;
	
	saveReplayHeader();
}

void saveReplayHeader()
{
	if (replayFile == NULL)
	{
		return;
	}

	rewind(replayFile);

	fwrite(&replayData.version, sizeof(replayData.version), 1, replayFile);
	fwrite(&replayData.seed, sizeof(replayData.seed), 1, replayFile);
	fwrite(&game.score, sizeof(replayData.score), 1, replayFile);

	replayData.seed = SWAP32(replayData.seed);
	game.score      = SWAP32(game.score);

	printf("Replay Version : %0.1f\n", replayData.version);
	printf("Replay Seed  : %d\n", replayData.seed);
	printf("Replay Score : %d\n", game.score);
}

void loadReplayHeader()
{
	float version = VERSION;
	
	if (replayFile == NULL)
	{
		return;
	}

	rewind(replayFile);

	fread(&replayData.version, sizeof(replayData.version), 1, replayFile);
	fread(&replayData.seed, sizeof(replayData.seed), 1, replayFile);
	fread(&replayData.score, sizeof(replayData.score), 1, replayFile);

	replayData.seed  = SWAP32(replayData.seed);
	replayData.score = SWAP32(replayData.score);

	printf("Replay Version : %0.1f\n", replayData.version);
	printf("Replay Seed    : %d\n", replayData.seed);
	printf("Replay Score   : %d\n", replayData.score);
	
	if (replayData.version != version)
	{
		printf("WARNING: Replay version is not the same as the software version!\n");
		printf("WARNING: This replay might not playback correctly!\n\n");
		printf("Software Version : %0.1f\n", version);
		printf("Replay Version   : %0.1f\n", replayData.version);
	}

	replayData.index = REPLAY_BUFFER;
}

void saveReplayData(int flush)
{
	if (replayFile == NULL)
	{
		return;
	}

	memcpy(replayData.input[replayData.index++], config.input, sizeof(config.input));

	if (replayData.index == REPLAY_BUFFER || flush == 1)
	{
		replayData.index = fwrite(replayData.input, sizeof(config.input), replayData.index, replayFile);

		if (flush == 1)
		{
			game.recording = 0;

			saveReplayHeader();

			fclose(replayFile);
		}

		replayData.index = 0;
	}
}

void loadReplayData()
{
	if (replayFile == NULL)
	{
		return;
	}

	if (replayData.index == REPLAY_BUFFER)
	{
		replayData.index = fread(replayData.input, sizeof(config.input), replayData.index, replayFile);

		if (replayData.index == 0)
		{
			printf("End of replay\n");

			game.replaying = 0;
		}

		replayData.index = 0;
	}

	memcpy(config.input, replayData.input[replayData.index++], sizeof(config.input));
}
